// The sample robotarm model.  You should build a file
// very similar to this for when you make your model.
#pragma warning (disable : 4305)
#pragma warning (disable : 4244)
#pragma warning(disable : 4786)

#include "particle.h"
#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include "particleSystem.h"


#include "floor.h"
#include "mat.h"

#include <FL/gl.h>
#include <stdlib.h>

// This is a list of the controls for the RobotArm
// We'll use these constants to access the values 
// of the controls from the user interface.
enum RobotArmControls
{ 
			BASE_ROTATION = 0,  UPPER_TILT ,  LOWER_TILT ,
			BASE_LENGTH,  UPPER_LENGTH, LOWER_LENGTH,PARTICLE_COUNT,
			NUMCONTROLS, 
};

void ground(float h);
void base(float h);
void upper_arm(float h);
void lower_arm(float h);
void y_box(float h);


// To make a RobotArm, we inherit off of ModelerView
class RobotArm : public ModelerView 
{
public:
    RobotArm(int x, int y, int w, int h, char *label) 
        : ModelerView(x,y,w,h,label) {}
    virtual void draw();
};

// We need to make a creator function, mostly because of
// nasty API stuff that we'd rather stay away from.
ModelerView* createRobotArm(int x, int y, int w, int h, char *label)
{ 
    return new RobotArm(x,y,w,h,label); 
}

// We'll be getting the instance of the application a lot; 
// might as well have it as a macro.
#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))


// We are going to override (is that the right word?) the draw()
// method of ModelerView to draw out RobotArm
void RobotArm::draw()
{
	// pick up the slider values
	// if we do this once we don't
	// have to worry about it again
	float pc = VAL( PARTICLE_COUNT );
	float theta = VAL( BASE_ROTATION );
	float phi = VAL( UPPER_TILT );	
	float psi = VAL( LOWER_TILT );
	float h1 = VAL( BASE_LENGTH );
	float h2 = VAL( UPPER_LENGTH );
	float h3 = VAL( LOWER_LENGTH );



	// get the particle system
	// it holds some of the functions that we
	// will use to place that same ps
	ParticleSystem *ps = ModelerApplication::Instance()->GetParticleSystem();

    
	// This call takes care of a lot of the nasty projection 
    // matrix stuff
    ModelerView::draw();

    // Save the camera transform that was applied by
    // ModelerView::draw() above.
    Mat4f matCam = ps->glGetMatrix( GL_MODELVIEW_MATRIX );
    

	// define the earth
	ground(-0.2);

	// define the model
    glRotatef( theta, 0.0, 1.0, 0.0 );		// turn the whole assembly around the y-axis. 
	base(h1);								// draw the base

    glTranslatef( 0.0, h1, 0.0 );			// move to the top of the base
    glRotatef( phi, 0.0, 0.0, 1.0 );		// rotate around the z-axis for the upper arm
	upper_arm(h2);							// draw the upper arm

    glTranslatef( 0.0, h2, 0.0 );			// move to the top of the upper arm
    glRotatef( psi, 0.0, 0.0, 1.0 );		// rotate  around z-axis for the lower arm
	lower_arm(h3);	
	
	// translate to the end of the arm and make particles come from it
	glPushMatrix();
		glTranslatef( 0.0, h3, 0.0 );
		
		// get the world point of the particle spawn point
		Mat4f particleXform = ps->getWorldXform( matCam); 
		
		// translate to the origin of particle space
		// (note that the Mat4X class will convert a 3 vector
		// to a homogenous point for multiplication)
		Vec3f particleOrigin = particleXform * Vec3f(0,0,0);

		// Note that you will have to define the particle
		// class yourself in our implementation this constructor
		// creates a particle that will dissappear after 5 seconds
		Particle *p = new Particle(particleOrigin, 5.0f);
		
		// add the particle to the particle system. You will
		// have to come up with some way of doing this for your
		// system as well.
		ps->addParticle(p);
	glPopMatrix();

	//*** DON'T FORGET TO PUT THIS IN YOUR OWN CODE **/
	endDraw();


}


void ground(float h) 
{
	glDisable(GL_LIGHTING);
	glColor3f(0.65,0.45,0.2);
	glPushMatrix();
	glScalef(30,0,30);
	y_box(h);
	glPopMatrix();
	glEnable(GL_LIGHTING);
}


void base(float h) {
    setDiffuseColor( 0.8, 0.6, 0.6 );
	setAmbientColor( 0.8, 0.6, 0.6 );
	glPushMatrix();
	glRotatef(-90,1.0,0.0,0.0);				// flip it up so we can draw along z
	drawCylinder( h, 1.0, 1.0 );
	glPopMatrix();
}

void upper_arm(float h) {					// draw the lower arm
	setDiffuseColor( 0.6, 0.8, 0.6 );
	setAmbientColor( 0.6, 0.8, 0.6 );
	y_box(h);
}

void lower_arm(float h) {					// draw the upper arm
	setDiffuseColor( 0.6, 0.6, 0.8 );
	setAmbientColor( 0.6, 0.6, 0.8 );
	y_box(h);
}



void y_box(float h) {

	glBegin( GL_QUADS );

	glNormal3d( 1.0 ,0.0, 0.0);			// +x side
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d( 0.25,0.0,-0.25);
	glVertex3d( 0.25,  h,-0.25);
	glVertex3d( 0.25,  h, 0.25);

	glNormal3d( 0.0 ,0.0, -1.0);		// -z side
	glVertex3d( 0.25,0.0,-0.25);
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d(-0.25,  h,-0.25);
	glVertex3d( 0.25,  h,-0.25);

	glNormal3d(-1.0, 0.0, 0.0);			// -x side
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d(-0.25,  h, 0.25);
	glVertex3d(-0.25,  h,-0.25);

	glNormal3d( 0.0, 0.0, 1.0);			// +z side
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d( 0.25,  h, 0.25);
	glVertex3d(-0.25,  h, 0.25);

	glNormal3d( 0.0, 1.0, 0.0);			// top (+y)
	glVertex3d( 0.25,  h, 0.25);
	glVertex3d( 0.25,  h,-0.25);
	glVertex3d(-0.25,  h,-0.25);
	glVertex3d(-0.25,  h, 0.25);

	glNormal3d( 0.0,-1.0, 0.0);			// bottom (-y)
	glVertex3d( 0.25,0.0, 0.25);
	glVertex3d(-0.25,0.0, 0.25);
	glVertex3d(-0.25,0.0,-0.25);
	glVertex3d( 0.25,0.0,-0.25);

	glEnd();
}

int main()
{
  

	 	// Initialize the controls
	// Constructor is ModelerControl(name, minimumvalue, maximumvalue, 
	//								 stepsize, defaultvalue)
    // You will want to modify this to accommodate your model.
    ModelerControl controls[NUMCONTROLS];   

    controls[BASE_ROTATION]  = ModelerControl( "base rotation (theta)", -180.0, 180.0, 0.1, 0.0 );
	controls[UPPER_TILT]     = ModelerControl( "upper arm tilt (phi)", -90.0, 90.0, 0.1, 0.0 );
   	controls[LOWER_TILT]     = ModelerControl( "lower arm tilt (psi)", -90.0, 90.0, 0.1, 0.0 ); 
   	controls[BASE_LENGTH]    = ModelerControl( "base height (h1)", 1, 10.0, 0.1, 2.0 );
    controls[LOWER_LENGTH]   = ModelerControl( "lower arm length (h3)", 1, 10.0, 0.1, 1.0 );
    controls[UPPER_LENGTH]   = ModelerControl( "upper arm length (h2)", 1, 10.0, 0.1, 3.0 );
	controls[PARTICLE_COUNT] = ModelerControl( "particle count (pc)", 0.0, 5.0, 0.1, 5.0 );

	ParticleSystem *ps = new ParticleSystem();

	// You should create a ParticleSystem object ps here and then
	// call ModelerApplication::Instance()->SetParticleSystem(ps)
	// to hook it up to the animator interface.

    ModelerApplication::Instance()->Init(&createRobotArm, controls, NUMCONTROLS);

	ModelerApplication::Instance()->SetParticleSystem(ps);
    return ModelerApplication::Instance()->Run();
}
